#ifndef _CHRONOPTICS_TOF_CSF_READER_HPP_
#define _CHRONOPTICS_TOF_CSF_READER_HPP_

#include <chronoptics/tof/csf_reader.h>

#include <chronoptics/tof/data.hpp>
#include <chronoptics/tof/stream.hpp>
#include <chronoptics/tof/calibration.hpp>
#include <chronoptics/tof/camera_config.hpp>

namespace chronoptics {
namespace tof {

/** DEPRECATED, use CSFCamera. This class reads csf files
*/
class CsfReader : public detail::Base<tof_csf_reader, tof_csf_reader_delete> {
 public:
  /** Construct from pointer */
  CsfReader(tof_csf_reader_t ptr = nullptr) {
    this->ptr_ = ptr;
  }

  /** Open a csf file for reading
   * @param file_location File location of the csf file
   */
  CsfReader(StringView file_location) {
    this->ptr_ = tof_csf_reader_new(file_location, TOF_ERROR_HANDLER{});
  }

  /** Read the next data frame from disk
   * @return A data frame
   */
  Data read_frame() {
    Data new_data(static_cast<tof_data_t>(nullptr));
    auto ptr = reinterpret_cast<tof_data_t*>(&new_data);
    *ptr = tof_csf_reader_read_frame(this->ptr_, TOF_ERROR_HANDLER{});
    return new_data;
  }

  /** Read the next data frame from disk into the supplied pointer
   * @param pointer The pointer to the data to write into
   * @param capacity The amount of data the pointer can hold
   * @param callback Callback that will be called when pointer is no longer in
   * use
   * @return A data frame
   */
  Data read_frame_into_pointer(uint8_t* pointer, size_t capacity, user_pointer_destructed_fn &callback) {
    Data new_data(static_cast<tof_data_t>(nullptr));
    auto ptr = reinterpret_cast<tof_data_t*>(&new_data);
    *ptr = tof_csf_reader_read_frame_into_pointer(this->ptr_, pointer, capacity, user_pointer_destructed_cb_handler, &callback, TOF_ERROR_HANDLER{});
    return new_data;
  }

  /** Read the next data frame from disk into the supplied pointer
   * @param pointer The pointer to the data to write into
   * @param capacity The amount of data the pointer can hold
   * @param callback Callback that will be called when pointer is no longer in
   * use
   * @param callback_user_data User data that will be passed back when the function pointer is called
   * @return A data frame
   */
  Data read_frame_into_pointer(uint8_t* pointer, size_t capacity, tof_user_pointer_destructed_t callback, void* callback_user_data) {
    Data new_data(static_cast<tof_data_t>(nullptr));
    auto ptr = reinterpret_cast<tof_data_t*>(&new_data);
    *ptr = tof_csf_reader_read_frame_into_pointer(this->ptr_, pointer, capacity, callback, callback_user_data, TOF_ERROR_HANDLER{});
    return new_data;
  }

  /** Get the total number of frames in the csf file
   * @return Total number of frames in the csf file
   */
  size_t number_of_frames() const {
    return tof_csf_reader_number_of_frames(this->ptr_, TOF_ERROR_HANDLER{});
  }

  /** Returns an empty Data class containing the header information for the
   * specified frame
   * @param index Which frame to get the header from
   * @return Empty Data instance containing header information
   */
  Data get_header(size_t index) const {
    Data new_data(static_cast<tof_data_t>(nullptr));
    auto ptr = reinterpret_cast<tof_data_t*>(&new_data);
    *ptr = tof_csf_reader_get_header(this->ptr_, index, TOF_ERROR_HANDLER{});
    return new_data;
  }

  /** Get the stream information of the next frame
   * @return Information on the next frame
   */
  Stream next_stream() const {
    Stream new_stream(static_cast<tof_stream_t>(nullptr));
    auto ptr = reinterpret_cast<tof_stream_t*>(&new_stream);
    *ptr = tof_csf_reader_next_stream(this->ptr_, TOF_ERROR_HANDLER{});
    return new_stream;
  }

  /** Get the size of the next frame
   * @return Size of the next frame
   */
  size_t next_frame_size() const {
    return tof_csf_reader_next_frame_size(this->ptr_, TOF_ERROR_HANDLER{});
  }

  /** Get the current frame position
   * @return Frame index
   */
  size_t frame_index() const {
    return tof_csf_reader_frame_index(this->ptr_, TOF_ERROR_HANDLER{});
  }

  /** Go to specified frame
   * @param index The index of the frame to jump to
   */
  void go_to_frame(size_t index) {
    return tof_csf_reader_go_to_frame(this->ptr_, index, TOF_ERROR_HANDLER{});
  }

  /** Get the streams that are in the csf file
   * @return Streams in the csf file
   */
  std::vector<Stream> streams() const {
    size_t size = tof_csf_reader_streams(this->ptr_, nullptr, 0, TOF_ERROR_HANDLER{});
    std::vector<Stream> vec;
    vec.reserve(size);
    for (size_t i = 0; i < size; i++)
      vec.emplace_back(static_cast<tof_stream_t>(nullptr));
    auto data = reinterpret_cast<tof_stream_t*>(vec.data());
    size = tof_csf_reader_streams(this->ptr_, data, vec.size(), TOF_ERROR_HANDLER{});
    return vec;
  }

  /** Get the number of frames contained in the file of the given stream
   * @param stream The stream
   * @return Number of frames for given stream
   */
  size_t stream_count(Stream &stream) const {
    return tof_csf_reader_stream_count(this->ptr_, *reinterpret_cast<tof_stream_t*>(&stream), TOF_ERROR_HANDLER{});
  }

  /** Go to the specified stream, the next read frame call will start from here
   * @param stream The stream
   * @param index The frame index
   */
  void go_to_stream(Stream &stream, size_t index) {
    return tof_csf_reader_go_to_stream(this->ptr_, *reinterpret_cast<tof_stream_t*>(&stream), index, TOF_ERROR_HANDLER{});
  }

  /** Get calibration from the csf file
   * @return Calibration
   */
  Calibration get_calibration() const {
    Calibration new_calibration(static_cast<tof_calibration_t>(nullptr));
    auto ptr = reinterpret_cast<tof_calibration_t*>(&new_calibration);
    *ptr = tof_csf_reader_get_calibration(this->ptr_, TOF_ERROR_HANDLER{});
    return new_calibration;
  }

  /** Get camera config from the csf file
   * @return Camera config
   */
  CameraConfig get_camera_config() const {
    CameraConfig new_camera_config(static_cast<tof_camera_config_t>(nullptr));
    auto ptr = reinterpret_cast<tof_camera_config_t*>(&new_camera_config);
    *ptr = tof_csf_reader_get_camera_config(this->ptr_, TOF_ERROR_HANDLER{});
    return new_camera_config;
  }

};

} // tof
} // chronoptics

#endif
